/* $NetBSD$ */

/*-
 * Copyright (c) 2009 Jared D. McNeill <jmcneill@invisible.ca>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE NETBSD FOUNDATION, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef _VDISK_VDISK_H
#define _VDISK_VDISK_H

#include <sys/types.h>
#include <sys/param.h>
#include <sys/disklabel.h>

#include <dev/pud/pud_msgif.h>

#define _PATH_PUD	"/dev/pud"

#define	VDISK_MAJOR		311
#define VDISK_NUM_PARTITIONS	16

#ifndef EPASSTHROUGH
#define EPASSTHROUGH	-4
#endif

struct vdisk_part;

struct vdisk_if {
	int	(*open)(void *, int, int);
	int	(*close)(void *, int, int);
	void	(*read)(void *, uint8_t *, off_t, size_t *);
	void	(*write)(void *, uint8_t *, off_t, size_t *);
	int	(*ioctl)(void *, dev_t, u_long, void *, int);
	int	(*sync)(void *);

	void	(*sizedev)(void *, uint64_t *);
};

struct vdisk_part {
	dev_t		vp_regdev;
	int		vp_fd;
	int		vp_part;
	struct vdisk	*vp_vd;
};

struct vdisk {
	void		*vd_cookie;
	struct pud_req	*vd_pdr;
	size_t		vd_pdrlen;
	struct vdisk_if	vd_if;
	char		vd_devdir[PATH_MAX];
	int		vd_unit;
	struct vdisk_part vd_vp[VDISK_NUM_PARTITIONS];
	struct disklabel vd_label;
};

struct vdisk *	vdisk_init(const char *, int, void *);
void		vdisk_fini(struct vdisk *);
void		vdisk_main(struct vdisk *);
void		vdisk_signal(int);

void		vdisk_getdisklabel(struct vdisk *);
int		vdisk_ioctl(struct vdisk_part *, u_long, void *, int);

int		vdisk_report(const int, const char *, ...);

#endif /* !_VDISK_VDISK_H */
